//*****************************************************************************
// Copyright (c) 2012, Dave Hein
// See end of file for terms of use.
//*****************************************************************************
#include <stdio.h>
#include <propeller.h>

typedef struct SpinixSerialS {
    int tm_seconds;
    int tm_clkticks;
    int bit_ticks;
    short rx_head;
    short rx_tail;
    short tx_head;
    short tx_tail;
    short rx_buffer;
    short tx_buffer;
    short rx_mask;
    short tx_mask;
    char cog;
    char lock;
    char rx_pin;
    char tx_pin;
    char rxtx_mode;
} SpinixSerialT;

#define pSerialAddress ((SpinixSerialT **)0x7fe0)

void rxflush(void)
{
    SpinixSerialT *pSerial = *pSerialAddress;
    pSerial->rx_tail = pSerial->rx_head;
}

int rxcheck(void)
{
    int rxbyte = -1;
    SpinixSerialT *pSerial = *pSerialAddress;
    int rx_tail = pSerial->rx_tail;

    if (rx_tail != pSerial->rx_head)
    {
        unsigned char *rx_buffer = (unsigned char *)(int)pSerial->rx_buffer;
        rxbyte = rx_buffer[rx_tail];
        pSerial->rx_tail = (rx_tail + 1) & pSerial->rx_mask;
    }

    return rxbyte;
}

int rx(void)
{
    int rxbyte;

    while ((rxbyte = rxcheck()) == -1);

    return rxbyte;
}

void txchar(int txbyte)
{
    SpinixSerialT *pSerial = *pSerialAddress;
    int tx_mask = pSerial->tx_mask;
    int tx_head = pSerial->tx_head;
    int tx_head1 = (tx_head + 1) & pSerial->tx_mask;
    unsigned char *tx_buffer = (unsigned char *)(int)pSerial->tx_buffer;

    while (pSerial->tx_tail == tx_head1);
    tx_buffer[tx_head] = txbyte;
    pSerial->tx_head = tx_head1;
}

void tx(int txbyte)
{
    SpinixSerialT *pSerial = *pSerialAddress;

    if (pSerial->rxtx_mode & 0x10)
    {
        int locknum = pSerial->lock - 1;
        while (lockset(locknum));
        txchar(txbyte);
        lockclr(locknum);
    }
    else
    {
        txchar(txbyte);
    }
}

static int _serial_putbyte(int c, FILE *fp)
{
    tx(c);
    return c;
}

static int _serial_getbyte(FILE *fp)
{
    return rx();
}

static int _serial_fopen(FILE *fp, const char *name, const char *mode)
{
    fp->drvarg[0] = (int)*pSerialAddress; // Save handle location
    fp->_flag |= _IODEV; // Mark it as being a terminal
    return 0;
}

static char _SpinixSerialPrefix[] = "SPINSER:";

_Driver _SpinixSerialDriver =
{
    _SpinixSerialPrefix,
    _serial_fopen,
    NULL,             // fclose
    _term_read,
    _term_write,
    NULL,             // seek
    NULL,             // remove
    _serial_getbyte,
    _serial_putbyte
};
/*
+-----------------------------------------------------------------------------+
|                       TERMS OF USE: MIT License                             |
+-----------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy |
|of this software and associated documentation files (the "Software"), to deal|
|in the Software without restriction, including without limitation the rights |
|to use, copy, modify, merge, publish, distribute, sublicense, and/or sell    |
|copies of the Software, and to permit persons to whom the Software is        |
|furnished to do so, subject to the following conditions:                     |
|                                                                             |
|The above copyright notice and this permission notice shall be included in   |
|all copies or substantial portions of the Software.                          |
|                                                                             |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   |
|IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     |
|FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE  |
|AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER       |
|LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,|
|OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE|
|SOFTWARE.                                                                    |
+-----------------------------------------------------------------------------+
*/
